<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla03200RestricaoDeProdutoPorGrupoDePrato extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $crudService;
    protected $parameterBag;
    protected $util;
    protected $dataSourceManager;
    protected $connection;
    protected $RestrContrService;
    protected $errorMessages = array();
    protected $errors = array();

	public function __construct($crudService, $parameterBag, $util, $dataSourceManager, $connection, $RestrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    //Cascade save Vigencia->Servico->Produto
    public function saveVigencia($vigencia, $response, $dataSourceName = "vigrestprodgr") {
        try {
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($vigencia));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrvigencia = $rowsToSave[0]["NRVIGENCIA"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrvigencia;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveRestprodgr($restprodgr, $response) {
        try {
            $dataSourceName = "restprodgr";
            $restprodgr["NRRESTPRODGR"] = $this->RestrContrService->getNewCode($dataSourceName);
            $dataSet = new DataSet($dataSourceName, array($restprodgr));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrrestprodgr = $rowsToSave[0]["NRRESTPRODGR"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrrestprodgr;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveProduto(DTO\Request $request, DTO\Response $response) {
        try {
            
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            
            foreach ($row["restprodgr"]["NMPRODUTO"] as $produto) {
                $restprodgrRow = array(
                    "NRVIGENCIA"       => isset($nrvigencia) ? $nrvigencia : $row["restprodgr"]["NRVIGENCIA"],
                    "NRGRUPOPRATOSERV" => $row["restprodgr"]["NRGRUPOPRATOSERV"],
                    "NRGRUPOPRATO"     => $row["restprodgr"]["NRGRUPOPRATO"],
                    "CDSERVFILI"       => $row["restprodgr"]["CDSERVFILI"],
                    "CDFILIAL"         => $row["restprodgr"]["CDFILIAL"],
                    "CDPRODUTO"        => $produto["CDPRODUTO"],
                    "__is_new"         => true
                );
                
                try {
                    $this->util->validateDuplicatedRecords($restprodgrRow, "restprodgr", array("CDPRODUTO", "NRGRUPOPRATOSERV", "NRVIGENCIA"));
                    $this->saveRestprodgr($restprodgrRow, $response);
                } catch(\Exception $e) {
                    if ($e->getMessage() == "registerExists") {
                        array_push($this->errorMessages, array("NMPRODUTO" => $produto["NMPRODUTO"]));
                    } else {
                        throw $e;
                    }
                }
            }

            $this->connection->commit();
            if (!empty($this->errorMessages)) {
                $response->addDataSet(new DataSet("errors", $this->errorMessages));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function deleteServicos(DTO\Request $request, DTO\Response $response) {
        try {
            $this->connection->beginTransaction();
            
            $rows = $request->getDataSet()->getRows();
            foreach ($rows as $row) {
                $this->deleteRestprodgr($row["NRVIGENCIA"], $row["CDSERVFILI"]);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];

            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }

            $nrvigenciaorig =  $row["NRVIGENCIAORIG"];
            $tpoper = $row["TPOPER"];
            foreach ($servicos as $servico) {
                if ($tpoper == "S") {
                    $this->deleteRestprodgr($nrvigenciadest, $servico["CDSERVFILI"]);
                }
                $produtos = $this->getRestprodgr($nrvigenciaorig, $servico["CDSERVFILI"]);
                $produtosExisting = $this->getRestprodgr($nrvigenciadest, $servico["CDSERVFILI"]);
                $produtosExisting = array_map(function($produto){
                    return $produto["CDPRODUTO"].'|'.$produto["NRGRUPOPRATO"];
                }, $produtosExisting);
                $produtosToSave = array();
                foreach ($produtos as &$produto) {
                    if (in_array($produto["CDPRODUTO"].'|'.$produto["NRGRUPOPRATO"], $produtosExisting)) {
                        continue;
                    } else {
                        $produto["NRRESTPRODGR"] = $this->RestrContrService->getNewCode("RESTPRODGR");
                        $produto["NRVIGENCIA"] = $nrvigenciadest;
                        $produto["__is_new"] = true;
                        array_push($produtosToSave, $produto);
                    }
                }
                $dataSourceName = "restprodgr";
                $dataSet = new DataSet($dataSourceName, $produtosToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function getRestprodgr($nrvigencia, $cdservfili) {
        $conditions = $this->RestrContrService->buildConditions(array("NRVIGENCIA", "CDSERVFILI"), array($nrvigencia, $cdservfili));
        return $this->crudService->getFilteredDataSet("restprodgr", $conditions)->getRows();
    }

    private function deleteRestprodgrByNrvigencia($nrvigencia) {
        $sql = "DELETE FROM RESTPRODGR WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    private function deleteRestprodgr($nrvigencia, $cdservfili) {
        $sql = "DELETE FROM RESTPRODGR WHERE NRVIGENCIA = :NRVIGENCIA AND CDSERVFILI = :CDSERVFILI AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "CDSERVFILI"  => $cdservfili,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function vigenciaExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();
            $vigenciaOrig = $row["VIGENCIAORIG"];
            $tables = $row["VIGENCIADEST"];
            foreach($tables as $table) {
                if (in_array(strtoupper($table), $this->RestrContrService->getTables())) { // Prevent SQL injection
                    $row = array(
                        "CDFILIAL" => $vigenciaOrig["CDFILIAL"],
                        "DTVIGENCIA" => $vigenciaOrig["DTVIGENCIA"],
                        "DTVIGENCIAFIM" => $vigenciaOrig["DTVIGENCIAFIM"],
                        "DESCRICAO" => $vigenciaOrig["DESCRICAO"],
                        "IDTIPORESTRICAO" => $vigenciaOrig["IDTIPORESTRICAO"],
                        "CDDESCOBSFILIAL" => $vigenciaOrig["CDDESCOBSFILIAL"],
                        "dataSourceName" => strtolower($table),
                        "fieldsToValidate" => array("CDFILIAL", "DTVIGENCIA"),
                        "__is_new" => true
                    );
                    try {
                        $this->RestrContrService->validateRegisterVigencia($row);
                        if (isset($row["DTVIGENCIAFIM"]) && strlen($row["DTVIGENCIAFIM"])) {
                            $this->RestrContrService->validateIntervalOverlap(null, $row["DTVIGENCIA"], $row["DTVIGENCIAFIM"], $row["dataSourceName"], $row["CDFILIAL"], $row["IDTIPORESTRICAO"]);
                        }
                        $this->saveVigencia($row, $response, $row["dataSourceName"]);
                    } catch( \Exception $e) {
                        if ($e->getMessage() == "registerExists") {
                            array_push($this->errors, $table);
                        } else {
                            throw $e;
                        }
                    }
                }
            }
            
            $this->connection->commit();
            if (!empty($this->errors)) {
                $response->addDataSet(new DataSet("errors", $this->errors));
                $response->setError(new Error("vigenciaExportErrors", 500));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}